import Flutter
import PhotoEditorSDK
import Foundation

protocol MyPhotoEditorDelegate {
    func isLogoStickerTool(_ value: Bool)
}

@available(iOS 9.0, *)
public class MyPhotoEditor: PhotoEditViewController {
    let podBundle = Bundle(for: SwiftPhotoEditorSdkPlugin.self) // for getting pod url
    var myDelegate: MyPhotoEditorDelegate!
    public override func viewDidLoad() {
        super.viewDidLoad()
        
    }
    
    private static let defaultTheme: Theme = {
        return Theme(backgroundColor: Colors.neutral50, menuBackgroundColor: .white, toolbarBackgroundColor: .white, primaryColor: Colors.primmary)
    }()
    
    private var theme = defaultTheme
    
    public override func menuViewController(_ menuViewController: MenuViewController, didSelect menuItem: MenuItem) {
        let teste = menuItem as? ToolMenuItem
        if teste?.title == "logos" {
            myDelegate.isLogoStickerTool(true)
        } else {
            myDelegate.isLogoStickerTool(false)
        }
        super.menuViewController(menuViewController, didSelect: menuItem)
    }
}

@available(iOS 9.0, *)
public class SwiftPhotoEditorSdkPlugin: NSObject, FlutterPlugin, MyPhotoEditorDelegate {
    var controller: UIViewController!
    var globalResult: FlutterResult!
    let podBundle = Bundle(for: SwiftPhotoEditorSdkPlugin.self) // for getting pod url
    private var menuItems = [PhotoEditMenuItem]()
    private var isLogoStickerTool = false
    static var registrarToSendMessages : FlutterPluginRegistrar? = nil
    
    func isLogoStickerTool(_ value: Bool) {
        isLogoStickerTool = value
    }
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        self.translate()
        if let licenseURL = Bundle.main.url(forResource: "license", withExtension: "") {
            PESDK.unlockWithLicense(at: licenseURL)
        }
        let channel = FlutterMethodChannel(name: "photo_editor_sdk", binaryMessenger: registrar.messenger())
        let instance = SwiftPhotoEditorSdkPlugin()
        registrarToSendMessages = registrar
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        controller = UIApplication.shared.keyWindow?.rootViewController
        globalResult = result
        let methodIdentifier = MethodIdentifierEnum(rawValue: call.method)
        switch(methodIdentifier) {
        case .hasAddLogo:
            if let args = call.arguments as? Dictionary<String, Any> {
                UserDefaults.standard.set(args["hasAddLogo"], forKey: "hasAddLogo")
                UserDefaults.standard.set(args["urlAddLogo"], forKey: "urlAddLogo")
            }
            break
        case .addAllContents:
            if let args = call.arguments as? Dictionary<String, Array<String>> {
                UserDefaults.standard.set(nil, forKey: "logos")
                UserDefaults.standard.set(nil, forKey: "stickers")
                UserDefaults.standard.set(args["logos"], forKey: "logos")
                UserDefaults.standard.set(args["stickers"], forKey: "stickers")
                setUpStickers()
            }
            break
        case .editImageSerialized:
            if let args = call.arguments as? Dictionary<String, String> {
                guard let image = args["pathUrl"],
                    let url = URL(string: image) else { return }
                do {
                    let data = try Data(contentsOf: url)
                    if let uiImage = UIImage(data: data) {
                        let photoImage = Photo(image: uiImage)
                        if let jsonSerr = args["jsonSerialized"]{
                            readSerialized(path: jsonSerr, photo: photoImage)
                        }
                    }
                } catch {
                    print(error)
                }
            }
            break
        case .editImageByUrl:
            guard let args = call.arguments as? String,
                let url = URL(string: args)
                else {
                    return
            }
            do {
                let data = try Data(contentsOf: url)
                if let uiImage = UIImage(data: data) {
                    let photoImage = Photo(image: uiImage)
                    self.presentPhotoEditViewController(photo: photoImage)
                }
            } catch {
                print(error)
            }
            break
        case .editImageByFile:
            guard let args = call.arguments as? String,
                let image = UIImage(named: args) else { return }
            let photoImage = Photo(image: image)
            self.presentPhotoEditViewController(photo: photoImage)
            break
        case .setUserId:
            let args = call.arguments as? String
            UserDefaults.standard.set(args, forKey: "userId")
        case .setEditorOptions:
            if let args = call.arguments as? Array<String> {
                createDefaultItems(selecteds: args)
            }
        default:
            break
        }
    }
    
    func readSerialized(path: String, photo: Photo){
        if let url = URL(string: path){
            URLSession.shared.dataTask(with: url) { (data, response, error) -> Void in
                if let dataNotNull = data, error == nil {
                    do {
                        let json = try JSONSerialization.jsonObject(with: dataNotNull, options: .mutableContainers) as? [String: Any]
                        self.presentPhotoEditViewControllerSerialized(photo: photo, json: dataNotNull)
                    } catch {
                        print("Unexpected error reading file.")
                    }
                }
            }.resume()
        }
    }
    private func presentPhotoEditViewControllerSerialized(photo: Photo, json: Data) {
        let inputImage: Photo = photo
        let data: Data = json
        let photo = inputImage
        let assetCatalog = AssetCatalog.sharedItems
        let deserializationResult = Deserializer.deserialize(data: data, imageDimensions: inputImage.size, assetCatalog: assetCatalog)
        if let model = deserializationResult.model {
            let configuration = buildConfiguration()
            let photoEditViewController = MyPhotoEditor(photoAsset: photo, configuration: configuration, photoEditModel: model)
            photoEditViewController.myDelegate = self
            photoEditViewController.modalPresentationStyle = .fullScreen
            photoEditViewController.delegate = self
            DispatchQueue.main.async {
                self.controller.present(self.createPhotoEditViewController(with: photo, and: model), animated: true, completion: nil)
            }
        }
        
    }
    
    private func presentPhotoEditViewController(photo: Photo) {
        DispatchQueue.main.async {
            self.controller.present(self.createPhotoEditViewController(with: photo), animated: true, completion: nil)
        }
    }
    
    private func createPhotoEditViewController(with photo: Photo, and photoEditModel: PhotoEditModel = PhotoEditModel()) -> PhotoEditViewController {
        let configuration = buildConfiguration()
        
        // Create a photo edit view controller
        let photoEditViewController = MyPhotoEditor(photoAsset: photo, configuration: configuration, photoEditModel: photoEditModel)
        photoEditViewController.myDelegate = self
        photoEditViewController.modalPresentationStyle = .fullScreen
        photoEditViewController.delegate = self
        
        
        return photoEditViewController
    }
    
    func createDefaultItems(selecteds: Array<String>) {
        guard let imageLogo = UIImage(named: "ico_logo", in: podBundle, compatibleWith: nil),
            let imageStickers = UIImage(named: "ico_sticker", in: podBundle, compatibleWith: nil),
            let imageFilter = UIImage(named: "ico_filter", in: podBundle, compatibleWith: nil)
            else {
                return
        }
        
        selecteds.forEach { (option) in
            let optionEnum = PhotoEditMenuEnum(rawValue: option)
            switch optionEnum {
                
            case .brush:
                menuItems.append(PhotoEditMenuItem.tool(ToolMenuItem.createBrushToolItem()!))
                break
            case .frame:
                menuItems.append(PhotoEditMenuItem.tool(ToolMenuItem.createFrameToolItem()!))
                break
            case .transform:
                menuItems.append(PhotoEditMenuItem.tool(ToolMenuItem.createTransformToolItem()!))
                break
            case .logo:
                let logoTool = ToolMenuItem(
                    title: "logos",
                    icon: imageLogo,
                    toolControllerClass: LogoStickerToolController.self)!
                menuItems.append(PhotoEditMenuItem.tool(logoTool))
                break
            case .sticker:
                let adesivosTool = PhotoEditMenuItem.tool(
                    ToolMenuItem(
                        title: "adesivos",
                        icon: imageStickers,
                        toolControllerClass: CustomStickerController.self)!
                )
                menuItems.append(adesivosTool)
                break
            case .text:
                menuItems.append(PhotoEditMenuItem.tool(ToolMenuItem.createTextToolItem()!))
                break
            case .textDesign:
                menuItems.append(PhotoEditMenuItem.tool(ToolMenuItem.createTextDesignToolItem()!))
                break
            case .adjust:
                menuItems.append(PhotoEditMenuItem.tool(ToolMenuItem.createAdjustToolItem()!))
                break
            case .filter:
                let filterTool = PhotoEditMenuItem.tool(
                    ToolMenuItem(
                        title: "filtro",
                        icon: imageFilter,
                        toolControllerClass: CustomFilterToolController.self)!
                )
                menuItems.append(filterTool)
                break
            case .none:
                break
            }
        }
    }
    
    private func configureTransformToolbar(builder: (ImglyKit.ConfigurationBuilder)) {
        builder.configureTransformToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "transformar")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
    }
    
    private func configureFrameToolbar(builder: (ImglyKit.ConfigurationBuilder)) {
        builder.configureFrameToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "moldura")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
        
        builder.configureFrameOptionsToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "quadro")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
    }
    
    private func configureBrushToolbar(builder: (ImglyKit.ConfigurationBuilder)) {
        builder.configureBrushToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "pincel")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
        
        builder.configureBrushColorToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "pincel")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
    }
    
    private func configureStickerToolbar(builder: (ImglyKit.ConfigurationBuilder)) {
        builder.configureStickerToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: self.isLogoStickerTool ? "logos" : "adesivos")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
        
        builder.configureStickerOptionsToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "opções")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
    }
    
    private func configureTextToolbar(builder: (ImglyKit.ConfigurationBuilder)) {
        builder.configureTextToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "editor")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
        
        builder.configureTextFontToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "opções")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
        
        builder.configureTextColorToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "opções")
                view.addSubview(label)
                
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
        
        builder.configureTextOptionsToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "opções")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
    }
    
    private func configureAdjustToolbar(builder: (ImglyKit.ConfigurationBuilder)) {
        builder.configureAdjustToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "opções")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
    }
    
    private func configureFilterToolbar(builder: (ImglyKit.ConfigurationBuilder)) {
        builder.configureFilterToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "opções")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
    }
    
    private func configureDesignTextToolbar(builder: (ImglyKit.ConfigurationBuilder)) {
        builder.configureTextDesignToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "opções")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
        
        builder.configureTextDesignOptionsToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "opções")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
    }
    
    private func buildConfiguration() -> Configuration {
        let configuration = Configuration { builder in
            // Configure editor
            builder.configurePhotoEditViewController { options in
                options.menuItems = menuItems
                
                options.titleViewConfigurationClosure = { view in
                    /// This will add and UILabel upside of title of the toolBar. Creating a layout more trustworthy with Ahazou.
                    let label = self.createTitleToolbarLabel(title: "editor")
                    view.addSubview(label)
                    self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
                }
                
                options.discardButtonConfigurationClosure = { button in
                    button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
                }
                
                options.applyButtonConfigurationClosure = { button in
                    /// Editing the download/save button of PhotoEditView
                    button.setImage(UIImage(), for: .normal)
                    button.setTitle("salvar", for: .normal)
                    button.titleLabel?.font = UIFont.systemFont(ofSize: 18, weight: .semibold)
                    button.titleEdgeInsets.right = 20
                    button.titleEdgeInsets.left = -20
                }
            }
            
            configureTextToolbar(builder: builder)
            configureStickerToolbar(builder: builder)
            configureBrushToolbar(builder: builder)
            configureFrameToolbar(builder: builder)
            configureTransformToolbar(builder: builder)
            configureAdjustToolbar(builder: builder)
            configureFilterToolbar(builder: builder)
            configureDesignTextToolbar(builder: builder)
            
            // Configure theme
            builder.theme = self.theme
        }
        
        return configuration
    }
    
    fileprivate func createTitleToolbarLabel(title: String) -> UILabel {
        let padding = UIEdgeInsets(top: 0, left: 10, bottom: 0, right: 0)
        /// The right and left button has 60 width. 60 + 60 = 120 but the "save" button has a -20 left padding so 120 + 20 = 140
        let rect = CGRect(x: 0, y: 0, width: UIScreen.main.bounds.width - 140, height: 44).inset(by: padding)
        let label = UILabel(frame: rect)
        label.textAlignment = .left
        label.text = title
        label.textColor = .black
        label.backgroundColor = .white
        
        
        label.font = UIFont.systemFont(ofSize: 28, weight: .bold)
        
        return label
    }
    
    fileprivate func applyConstraintsOnTitleToolbarLabel(label: UILabel, view: UIView) {
        label.topAnchor.constraint(equalTo: view.topAnchor, constant: 0).isActive = true
        label.bottomAnchor.constraint(equalTo: view.bottomAnchor, constant: 0).isActive = true
        label.rightAnchor.constraint(equalTo: view.rightAnchor, constant: 0).isActive = true
        label.leftAnchor.constraint(equalTo: view.leftAnchor, constant: 0).isActive = true
    }
    
    static func preferredLocale() -> Locale {
        guard let preferredIdentifier = Locale.preferredLanguages.first else {
            return Locale.current
        }
        return Locale(identifier: preferredIdentifier)
    }
    
    fileprivate static func translate() {
        PESDK.localizationBlock = { stringToLocalize in
            return NSLocalizedString(stringToLocalize, tableName: "MyStrings", bundle: Bundle(for: SwiftPhotoEditorSdkPlugin.self), comment: "")
        }
    }
    
    private static let defaultTheme: Theme = {
        return Theme(backgroundColor: Colors.neutral50, menuBackgroundColor: .white, toolbarBackgroundColor: .white, primaryColor: Colors.primmary)
    }()
    
    private var theme = defaultTheme
}

@available(iOS 9.0, *)
extension SwiftPhotoEditorSdkPlugin: PhotoEditViewControllerDelegate {
    public func photoEditViewController(_ photoEditViewController: PhotoEditViewController, didSave image: UIImage, and data: Data) {
        if let navigationController = photoEditViewController.navigationController {
            navigationController.popViewController(animated: true)
        } else {
            if (data.count == 0) {
                globalResult(image.pngData())
            } else {
                globalResult(data)
            }
            
            controller.dismiss(animated: true, completion: nil)
        }
    }
    
    public func photoEditViewControllerDidFailToGeneratePhoto(_ photoEditViewController: PhotoEditViewController) {
        if let navigationController = photoEditViewController.navigationController {
            navigationController.popViewController(animated: true)
        } else {
            controller.dismiss(animated: true, completion: nil)
        }
    }
    
    public func photoEditViewControllerDidCancel(_ photoEditViewController: PhotoEditViewController) {
        if let navigationController = photoEditViewController.navigationController {
            navigationController.popViewController(animated: true)
        } else {
            controller.dismiss(animated: true, completion: nil)
        }
    }
}
